
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.


require("dialog")
require("map_methods")

--This file handles all the choices that you see in any state.
--It captures keyboard events and does not pass them to the parent state.
--It also calls the parent's draw function before its own to draw the backgrounds.

local last_draw, current_message, choices, current_choice, xpos, ypos, halign, valign

local blinktime = 0
local blink = false

choice_active = false

function move_choice(direction)
    for k, v in pairs(choices) do
        if v[1] == current_choice then
            local tmpk = k+direction
            if tmpk < 1 then --Loop around
                tmpk = (#choices)
            elseif tmpk > (#choices) then -- Loop around the other way
                tmpk = 1
            end
            current_choice = choices[tmpk][1]
            break
        end
    end
end

local exit_method, exit_args = nil, nil

function choice_last_draw()
	for k,v in pairs(previous_callbacks) do
		if v["draw"] then
			if v["draw"] ~= choice_draw then
				v["draw"]()
			end
		end
	end
end

function choice_enter(choices_table, default, xoffs, yoffs, horizalign, vertalign, exitcallback, exitcallbackargs)
    --"Enters" the state
    --Sets up all the callbacks and such.
    --Call this when you want to move to this state.
    --Including a callback will cause that to be called when the state exits
    --print("entering choice")
    choices = choices_table
    current_choice = default
    xpos = xoffs
    ypos = yoffs
    halign = horizalign
    valign = vertalign
    
    choice_active = true
    
    exit_method, exit_args = exitcallback, exitcallbackargs
    
    table.insert(previous_callbacks, deepcopy(callbacks)) --Save the previous callbacks list
    --print_table(previous_callbacks)
    if callbacks["draw"] ~= choice_draw then
		--last_draw = callbacks["draw"]
		--print("Last draw was a choice, no good.")
	end
	--print(last_draw)
    callbacks = {}
    callbacks["keypressed"] = choice_keypressed
    callbacks["keyreleased"] = choice_keyreleased
    callbacks["update"] = choice_update
    callbacks["draw"] = choice_draw
    callbacks["exit"] = choice_exit
    
end

function choice_exit()
    --The exit callback should NOT be called when entering another state, 
    --only when explicitly exiting and falling back to the previous state.
    
    --Restore our callbacks when we're done.
    --print("exiting choice")
    callbacks = deepcopy(previous_callbacks[#previous_callbacks])
    table.remove(previous_callbacks)

    if exit_method ~= nil then
        if type(exit_method) == "table" then
            print_table(exit_method)
        end
        
        if exit_args ~= nil then
            print(exit_args)
            exit_method(unpack(exit_args))
        else
            exit_method()
        end
        
    end
    
    choice_active = false
    
end

function choice_init()
    --Do your setup things here.
    --This should consist of things that only need to be called once per instance of the game.
    --Things like globals and such.
    --If anything needs to be done every time the state starts up, put it in choice_enter().
    
    love.graphics.setFont(love.graphics.newFont(tilesize/2))
    
    frame = load_dialog_frame("/gfx/frames/fantasy_bold.png") --Using the dialog frame for the choice's frame
    
end

function choice_keypressed(key, unicode)
    if key == " " then
        --Pressed space, so do something if you want.
        
        --Call the function for the current choice, and unpack the third as a table into the arguments for the function
        for k, v in pairs(choices) do
            if v[1] == current_choice then
                --print("calling")
                v[2](unpack(v[3]))
                --print("done calling")
                callbacks.exit()
            end
        end
        --And we're all done here so exit the choice.
        --print("Done with choice.")
        
    elseif key == "up" then
        --Go back one in the list
        move_choice(-1)
    elseif key == "down" then
        --Go forward one in the list
        move_choice(1)
    end
    
    if debug_on then
        if key == "p" then
            --Pressed p, so do something, but only if debugging is on.
            print("pressed p while in the choice state!")
        end
    end
end

function choice_keyreleased(key, unicode)
    if key == " " then
        --They released space!
    end
end

function choice_update(dt)
    blinktime = blinktime + dt
    if blinktime > 0.5 then
        blinktime = 0
        blink = not(blink)
    end
    --Just like LOVE's update method.
    --In other words, whatever you want to run every frame.
end

function choice_draw()
    choice_last_draw()
    --if cameracenter_translate then
        --love.graphics.translate(-cameracenter_translate[1], -cameracenter_translate[2])
    --end
    
    --Draw the frame (using dialog's frame here because it is general enough)
    local lineheight = love.graphics.getFont():getLineHeight()
    local linewidth = love.graphics.getFont():getWidth(current_choice)
    
    local len = 0
    local maxwidth = 0
    for k,v in pairs(choices) do
        len = len+ 1
        local w = love.graphics.getFont():getWidth(v[1])
        if w > maxwidth then
            maxwidth = w
        end
    end
    
    local item_spacing = tilesize/1.5
    
    local frameheight = (lineheight*len)+(len*item_spacing)+(item_spacing/2)
    local framewidth = maxwidth*2
    local x, y
    
    if halign == "right" then
        x = windowwidth-framewidth-xpos
    else 
        x = xpos
    end
    
    if valign == "bottom" then
        y = windowheight-frameheight-ypos
    else 
        y = ypos
    end
    
    draw_dialog_frame(frame, x, y, framewidth, frameheight)
    
    --Draw the choices
    --TODO: This should be a game setting.
    love.graphics.setColor(255, 255, 255)
    
    --These two are offsets from the top and left
    itemsy = y+(tilesize/4)
    itemsx = x+(tilesize/1.5)
    
    for k,v in pairs(choices) do
        love.graphics.print(v[1], itemsx, itemsy)
        itemsy = itemsy+item_spacing
    end
    
    --Draw the blinking cursor to the left of the current choice
    if blink == true then
        for k, v in pairs(choices) do
            if v[1] == current_choice then
                --FIXME: Only works for vertical choice windows
                local newy = (y+(tilesize/4))+((k-1)*(item_spacing))
               love.graphics.draw(cached_load_image("/gfx/downarrow.png"), itemsx-(tilesize/2.5), newy, -12, 0.5, 0.5)
               break
            end
            
        end
        
    end
    
    
    --local draw_retval = {draw_dialog_frame(frame, 0, tilesize*7.5, windowwidth, windowheight-(tilesize*7.5))}
    --local corner_height, corner_width, corner_sy, corner_sx, dialog_x, dialog_y, dialog_w, dialog_h, horizscale, vertscale = unpack(draw_retval)
    --And finally our text:
    --love.graphics.setColor(255, 255, 255)
    --local lineheight = love.graphics.getFont():getLineHeight()
    --local linewidth = love.graphics.getFont():getWidth(current_choice)
    --love.graphics.print(current_message, corner_width*corner_sx*1.25, dialog_y+(corner_height*corner_sy*3))
    
    --Maybe skip the whole UI library altogether and just draw our corners and sides manually
    --And then draw some text.
    --We'd need some logic up in the update function to break text into separate dialogs.
    
    
end
