
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.


function load_dialog_frame(filename)
    --Load our frame graphics
    --Frames should resemble:
    --CBBC
    --BNNB
    --BNNB
    --CBBC
    --Where C is a corner, B is a border, and N is the iNside of the dialog.
    --The actual image can be any size as long as the proportions are maintained. (I think.)
    local img = love.image.newImageData(filename)
    
    local frame = {}
    
    local f_height, f_width = img:getHeight(), img:getWidth()
    local f_cornerw, f_cornerh = f_height/4, f_width/4
    local f_border_vw, f_border_vh = f_height/4, f_width/2
    local f_centerw, f_centerh = f_height/2, f_height/2
    
    --Corners    
    frame.topleft = love.image.newImageData(f_cornerw, f_cornerh)
    frame.topleft:paste(img, 0, 0, 0, 0, f_cornerw, f_cornerh)
    
    frame.topright = love.image.newImageData(f_cornerw, f_cornerh)
    frame.topright:paste(img, 0, 0, f_width-f_cornerw, 0, f_cornerw, f_cornerh)
    
    frame.bottomleft = love.image.newImageData(f_cornerw, f_cornerh)
    frame.bottomleft:paste(img, 0, 0, 0, f_height-f_cornerh, f_cornerw, f_cornerh)
    
    frame.bottomright = love.image.newImageData(f_cornerw, f_cornerh)
    frame.bottomright:paste(img, 0, 0, f_width-f_cornerw, f_height-f_cornerh, f_cornerw, f_cornerh)
    
    --Borders
    frame.left = love.image.newImageData(f_border_vw, f_border_vh)
    frame.left:paste(img, 0, 0, 0, f_cornerh, f_border_vw, f_border_vh)
    
    frame.right = love.image.newImageData(f_border_vw, f_border_vh)
    frame.right:paste(img, 0, 0, f_width-f_cornerw, f_cornerh, f_border_vw, f_border_vh)
    
    frame.top = love.image.newImageData(f_border_vh, f_border_vw)
    frame.top:paste(img, 0, 0, f_cornerw, 0, f_border_vw, f_border_vh)
    
    frame.bottom = love.image.newImageData(f_border_vh, f_border_vw)
    frame.bottom:paste(img, 0, 0, f_cornerw, f_height-f_cornerh, f_border_vw, f_border_vh)
    
    --Center
    frame.center = love.image.newImageData(f_centerw, f_centerh)
    frame.center:paste(img, 0, 0, f_cornerw, f_cornerh, f_centerw, f_centerh)
    
    for k,v in pairs(frame) do
        local newimg = love.graphics.newImage(v)
        newimg:setFilter("linear", "nearest")
        frame[k] = newimg
    end
    
    return frame
end

function draw_dialog_frame(frame, x, y, w, h)
    --This should only be called in a Draw callback.
    
    --Draw our dialog background:
    local corner_height, corner_width = frame.topleft:getHeight(), frame.topleft:getWidth()
    local corner_sy, corner_sx = (tilesize/4)/corner_height, (tilesize/4)/corner_width
    
    --FIXME: Changing the dialog's size totally breaks this. Low priority, but would be nice to expand it if we wanted.
--~     local dialog_x, dialog_y = 0, tilesize*7.5
--~     local dialog_x, dialog_y = x, y
--~     local dialog_w, dialog_h = windowwidth, windowheight-tilesize
--~     local dialog_w, dialog_h = w, h
    
    local horizscale = (w/corner_width-(corner_width/2))
    --local vertscale = (((h+y)/corner_height)/(corner_sy*2))-4 --Old vertscale, leaving it here in case the new one is horribly broken.
    local vertscale = (h/corner_height-(corner_height/2))
    
    love.graphics.draw(frame.center, x+(corner_width*corner_sx), y+(corner_height*corner_sy), 0, horizscale/2, vertscale/2)
    
    love.graphics.draw(frame.top, x+(corner_width*corner_sx), y, 0, horizscale, corner_sx)
    
    love.graphics.draw(frame.bottom, x+(corner_width*corner_sx), (y+h)-(corner_height*corner_sy), 0, horizscale, corner_sx)
    
    love.graphics.draw(frame.left, x, y+(corner_height*corner_sy), 0, corner_sx, vertscale/2)
    
    love.graphics.draw(frame.right, (x+w)-(corner_width*corner_sx), y+(corner_height*corner_sy), 0, corner_sx, vertscale/2)
    
    love.graphics.draw(frame.topleft, x, y, 0, corner_sy, corner_sx)
    love.graphics.draw(frame.topright, (x+w)-(corner_width*corner_sx), y, 0, corner_sy, corner_sx)
    love.graphics.draw(frame.bottomleft, x, (y+h)-(corner_height*corner_sy), 0, corner_sy, corner_sx)
    love.graphics.draw(frame.bottomright, (x+w)-(corner_width*corner_sx), (y+h)-(corner_height*corner_sy), 0, corner_sy, corner_sx)
    
    return corner_height, corner_width, corner_sy, corner_sx, x, y, w, h, horizscale, vertscale
end

function draw_dialog_text(message, corner_width, corner_height, corner_sx, corner_sy, dialog_y)
    love.graphics.print(message, corner_width*corner_sx*1.25, dialog_y+(corner_height*corner_sy*3))
end
