
--Formulas defines a few functions:
	--"calc_hp" which calculates a monster's maximum HP
	--"calc_attack" which calculates damage dealt to target monster if monster used the given move.
	--"calc_exp" which calculates the experience points gained after battle.
	--"exp_to_level" which calculates the experience points required to gain a level.

require("monster_formulas")

function calc_hp(monster)
	--This is the "old" HP formula
	local base = monster.basestats.hp
	local level = monster.level
	local iv = monster.ivs.hp
    local ev = monster.evs.hp
    return (((iv+base+(math.sqrt(ev)/8)+50)*level)/50)+10
end

function calc_stat(monster, stat)
	local base = monster.basestats[stat]
	local level = monster.level
	local iv = monster.ivs[stat]
    local ev = monster.evs[stat]
    return (((iv+base+(math.sqrt(ev)/8))*level)/50)+5
end

function calc_attack(monster, targetmonster, move)
	--monster.attack could be monster.special if the attack is a special attack
	
	local stab = 1.0
	for k,m in pairs(move.types) do
        for _, value in pairs(monster.types) do
            if m == value then 
                stab = 1.5
                break
            end
        end
	end
    
	effectiveness = 1.0 
	
	local d = 2
	d = d * monster.level
	d = d + 10
	d = d * calc_stat(monster, 'attack')
	d = d * move.power
	d = d / (250*calc_stat(targetmonster, 'defense'))
	d = d + 2
	d = d * effectiveness
	d = d * stab
	d = d * (math.random(85, 100)/100.0)
	return math.floor(d)
    
end

function calc_exp(fainted_monster, winning_monster, num_winning_monsters)
	local s = num_winning_monsters --The number of monsters who survived combat and won.
	
	local a = 1--1 if wild, 1.5 if owned
	if (fainted_monster.originaltrainer ~= nil) then
		a = 1.5
	end
	
	local t = 1 --1 if OT is current owner, 1.5 if from domestic trade, 1.7 if from international trade
	
	local b = fainted_monster.expyield --The exp the fainted monster's species yields
	
	local e = 1 --1.5 if holding an exp boosting item
	
	local L = fainted_monster.level --The level of the fainted monster
	
	return round((a*t*b*e*L)/(7*s))
end

function exp_to_level(monster)
	--Each species of monster has this defined using the "exp_*" functions below
	return monster.exp_to_level(monster.level+1)
end

--The xp_* functions are functions used to calculate XP and are only used in this example and not required to be defined.
function exp_fast(level)
--[=[= 

>=exp_fast(100)
800000

=]=]
	return (4*math.pow(level, 3))/5
end

function exp_medfast(level)
--[=[= 

>=exp_medfast(100)
1000000

=]=]
	return math.pow(level, 3)
end

function exp_medslow(level)
--[=[= 

>=exp_medslow(100)
1059860

=]=]
	return (6/5)*math.pow(level, 3) - 15*math.pow(level, 2) + 100*level - 140
end

function exp_slow(level)
--[=[= 

>=exp_slow(100)
1250000

=]=]
	return (5*math.pow(level, 3))/4
end
