
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.



--This file is full of handy helper functions that should have been present in lua from the start :D

function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

function table_print (tt, indent, done)
  done = done or {}
  indent = indent or 0
  if type(tt) == "table" then
    for key, value in pairs (tt) do
      io.write(string.rep (" ", indent)) -- indent it
      if type (value) == "table" and not done [value] then
        done [value] = true
        io.write(string.format("[%s] => table\n", tostring (key)));
        io.write(string.rep (" ", indent+4)) -- indent it
        io.write("(\n");
        table_print (value, indent + 7, done)
        io.write(string.rep (" ", indent+4)) -- indent it
        io.write(")\n");
      else
        io.write(string.format("[%s] => %s\n",
            tostring (key), tostring(value)))
      end
    end
  else
    io.write(tt .. "\n")
  end
end

print_table = table_print --SanitySaver :D

function eval(str, noprint)
    if not(noprint) then
        print("Evaluating:", str)
    end
   return assert(loadstring(str))()
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function in_table(needle, haystack)
    for k,v in pairs(haystack) do
        if v == needle then
            return true
        end
    end
    return false
end


images = {} --Individual Sprites/Tiles
imagedata = {} --Whole images

function cached_load_image(filename)
    local img
    if images[filename] ~= nil then
        img = images[filename]
    else
        img = love.graphics.newImage(filename)
        images[filename] = img
    end
    return img
end

function load_from_spritesheet(sheetname, height, width, x, y)
    --Spritesheet is a filename to a single image full of sprites.
    --Height and Width are the h/w of the sprite you want.
    --X and Y are the top left corner of the sprite you want.
    -- TODO: Make this return Quads so we dont have to do all that paste()ing
    local img
    if imagedata[sheetname] ~= nil then
        img = imagedata[sheetname]
    else
        img = love.image.newImageData(sheetname)
        imagedata[sheetname] = img
    end
    local new = love.image.newImageData(width, height)
    new:paste(img, 0, 0, x*width, y*height, width, height)
    local newimg = love.graphics.newImage(new)
    newimg:setFilter("linear", "nearest")
    return newimg
end

function load_from_spritesheet_by_number(sheetname, height, width, num)
    --Given a number, it returns an Image
    local img = cached_load_image(sheetname)
    local y = math.floor((num*width)/img:getWidth())
    local x = num%(img:getWidth()/width)
    return load_from_spritesheet(sheetname, height, width, x, y)
end
