
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.


--System Libraries
require("string")
require("math")
math.randomseed(os.time()); math.random(); math.random();

--If this is turned on, it generates a profiling file, but it slows everything down.
--~ require("profiler")
--~ profiler.start()

--Love Libraries
require("AnAL")

--Project Libraries
require("colliderects")
require("map_loader")
require("tablesave")
require("helper_functions")

--Game definition files
--TODO: Set game folder from a command line argument or maybe a menu
gamesdir = "/games/"
gamename = "sample"
require(gamesdir..gamename.."/main.lua")

--Project state files
require("walkabout")
require("combat")
require("walkabout_dialog")



debug_on = true
--~ debug_on = false

function love.load()
    windowheight = love.graphics.getHeight()
    windowwidth = love.graphics.getWidth()
    print(windowheight, windowwidth)
    
    love.filesystem.setIdentity("crpg_"..gamename)
    
    love.graphics.setIcon(cached_load_image("gfx/capsule_fullsize.png"))
        
    love.graphics.setBackgroundColor(0,0,0)
    
    sounds = {}
    sounds["whitenoise"] = love.audio.newSource("/sfx/noise/white_noise.ogg", "static")

    --Effects need to be defined somewhere else.
    effects = {}
    effects["combatstartwild"] = effect_combat_start_wild --FIXME: This function is not defined anywhere yet
    

    callbacks = {}
    
    --State enter methods should push the callbacks variable here before they assign their own.
    --State exit methods should pop the callbacks variable from here and set the result to callbacks
    previous_callbacks = {} 
    
    combat_init()
    
    
    walkabout_init()
    walkabout_dialog_init()
    walkabout_enter() --Start with the walkabout state
    
end

function global_keypressed(key, unicode)
    if key == "escape" then
        --Drop out of the last state and move back to the previous one.
        if #previous_callbacks  > 0 then
            callbacks.exit()
        end
        
        if #previous_callbacks  == 0 then
            if profiler ~= nil then
                profiler.stop()
            end
            love.event.push('q')
        end
        
    elseif key == "d" then
        debug_on = not (debug_on)
    elseif key == "c" then
        local screenshot = love.graphics.newScreenshot()
        local data = love.image.newEncodedImageData( screenshot, "bmp")
        local ssname = os.time()
        --TODO: Set the screenshot directory based on the game we have loaded
        local file = love.filesystem.newFile("screenshot_"..ssname..".bmp")
        if file:open("w") then
            if file:write(data) then
                print("Screenshot saved to screenshot_"..ssname..".bmp")
            else
                print("Couldnt write to screenshot_"..ssname..".bmp :(")
            end
            file:close()
        else
            print("Couldnt open screenshot for writing :(")
        end
    elseif key == "s" then
        --Save our game.
        --Save any global things that need saving
        --The "player" table should be what the games define and we should save.
        --If you want something saved, tack it on to "player"
        
        --Update the player's stored location.
        player.locx = playerx
        player.locy = playery
        player.direction = player_dir
        
        if love.filesystem.write("savegame_"..os.time(), table.save(player)) then
            print("Save successful!")
        end
        
    elseif key == "l" then
        --Load the latest savefile.
        --FIXME: This loads every save file in order.
        for i, v in ipairs(love.filesystem.enumerate("/")) do
            if love.filesystem.isFile("/"..v) then
                if string.find(v,"savegame_") ~= nil then
					player = table.load(love.filesystem.read(v))

                    playerx = player.locx
                    playery = player.locy
                    load_map(gamesdir..gamename.."/maps/"..player.map..".tmx")
                    player_dir = player.direction
                    print_table(player.mapvars)
                end
            end
        end
    end
    
end

function global_keyreleased(key, unicode)
    --Dont need to do anything globally
end

function global_update(dt)
    --Dont need to do anything globally
end

function global_draw() 
    love.graphics.setScissor(0, 0, windowwidth, windowheight)
end

function love.keypressed(key, unicode)
    global_keypressed(key, unicode)
    if callbacks.keypressed then
        callbacks.keypressed(key, unicode)
    end
end

function love.keyreleased(key, unicode)
    global_keyreleased(key, unicode)
    if callbacks.keyreleased then
        callbacks.keyreleased(key, unicode)
    end
end

function love.update(dt)
    global_update(dt)
    if callbacks.update then
        callbacks.update(dt)
    end
end

function love.draw()
    love.graphics.push()
    
    global_draw()
    if callbacks.draw then
        callbacks.draw()
    end
    
    love.graphics.pop()


    if (debug_on == true) then
        --Finally, draw an fps counter. Its accuracy is debated however.
        love.graphics.print("FPS: "..love.timer.getFPS(), 10, 30)
    
        --And our version number. This is not automatically updated. Please update when you make a release.
        love.graphics.print("v0.0.2", 10, 60)
    end

    love.graphics.setColor(255, 255, 255) --Make sure our graphics come out white after goo
end

