
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.


--This file contains all the things that maps would want to call. Or most of them anyway.

--TODO: Should probably import the game's map methods, so we can let them define their own.

--TODO:We'll also probably need an event queue.

function map_save(varname, value)
    player[varname] = value
end

function map_load(varname, default)
    local retval = default
    if player[varname] ~= nil then
        retval = player[varname]
    end
    return retval
end

mapmethod_queue = {}
mapmethod_queue_lock = false
function queue_method(function_name, args)
    table.insert(mapmethod_queue, {function_name, args})
end

function do_multiple(methods_and_args)
    --Iterate methods and args, every odd entry is a function to call, every even entry is the arguments for that function.
    --These are all called at once and will not wait for blocking calls, so ordering doesnt matter.
    --This is handy for callbacks where you want to do more than one thing at once like a dialog and a sound effect.
    for i=1,#methods_and_args,2 do
        if methods_and_args[i] ~= nil then
            if methods_and_args[i+1] ~= nil then
                methods_and_args[i](unpack(methods_and_args[i+1]))
            else
                methods_and_args[i]()
            end
        end
    end
end

function do_wild(mons)
    --TODO: Implement via swapping out the love callbacks for the ones we want
    --mons should be a table of tables, consisting of:
    --monster species ID, level_min, level_max, percentage_rate
    --The level range is inclusive.
    --Remember to leave space in percentage_rate so that the player doesnt get overwhelmed in monsters.
    --Alternatively, if percentage_rate can be one of 5 constants will result in odds as below:
    --"verycommon": 1-in-18.5
    --"common": 1-in-22
    --"semirare": 1-in-28
    --"rare": 1-in-56
    --"veryrare": 1-in-150
    print(mons)
    
    --Set what the player will fight next
    --Call an effect, this will modify the draw callback so that its things get drawn.  Also give the effect the enter method you want it to call when its effect is over.
    enemy = {}
    enemy.monsters = {}
    enemy.monsters[1] = monsters.puppy()
    combat_enter()
end

function teleport(targetmap, x, y, centercamera)
    --Loads a given targetmap, places the player at a given x/y coord and optionally centers the camera.
    print("Teleporting you (for real this time) to...", targetmap, x, y, centercamera)
    load_map(gamesdir..gamename.."/maps/"..targetmap..".tmx")
    --playerx = x*tilesize
    --playery = y*tilesize
    --cameracenter = centercamera
    
    if playerObj ~= nil then
        playerObj:teleport(targetmap, x, y, centercamera)
    end
    
    --Store some stuff so we can save it.
    player.map = targetmap
    player.locx = playerx
    player.locy = playery
end

queued_dialogs = {}

function dialog(message, callback, callbackargs)
    --This call is long running, so lock the queue
    mapmethod_queue_lock = true 
    print("Dialog:", message)
    table.insert(queued_dialogs, message)
    walkabout_dialog_enter(callback, callbackargs)
end

function showeffect(effect)
    print("Effect:", effect)
end

function sound(soundobj, loop, volume, pitch)
    --Plays a given sound love.audio.Source, loop is a boolean determining if it should be looped.
    --Automatically plays the given sound. Use the soundobj you're calling this with to control it manually.
    --"volume" and "pitch" must be a number.
    --Returns the soundobj you gave for convenience.
    print("Sound:", soundobj, loop, volume, pitch)
    if type(soundobj) == "userdata" then
        if type(pitch) == "number" then
            soundobj:setPitch(pitch)
        else
            soundobj:setPitch(1)
        end
        
        if type(volume) == "number" then
            soundobj:setVolume(volume)
        else
            soundobj:setVolume(1)
        end
        
        if type(loop) == "boolean" then
            soundobj:setLooping(loop)
        else
            soundobj:setLooping(false)
        end
        
        love.audio.play(soundobj)
        
    elseif type(soundobj) == "string" then
        --Then load the file into the sounds array and play it
    end
    
    return soundobj
end

function stopsound(soundobj)
    --Stops a given soundobj (a love.audio.Source object)
    if type(soundobj) == "userdata" then
        love.audio.stop(soundobj)
    end
end
