
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.


require("choice")

--FIXME: Use the choice UI element instead of the homebrew one below

function menu_enter(menuitems, m_type, x_pos, y_pos, m_width, m_height)
    --"Enters" the state
    --Sets up all the callbacks and such.
    --Call this when you want to move to this state.
    --Arguments: 
        --menuitems, A table of numeric keys and values of tables containing: text lable and function to call when activated.
        --m_type, A string denoting which type of menu it should be: "wide", "tall", or "square".
    
    table.insert(previous_callbacks, deepcopy(callbacks)) --Save the previous callbacks list
    
    callbacks["keypressed"] = menu_keypressed
    callbacks["keyreleased"] = menu_keyreleased
    callbacks["update"] = menu_update
    last_draw = callbacks["draw"]
    callbacks["draw"] = menu_draw
    callbacks["exit"] = menu_exit
    
    --TODO: Store any old menu_items and menu_types so we can call menus from menus
    menu_items = menuitems
    menu_current_selection = 1
    menu_x, menu_y = x_pos, y_pos
    menu_width, menu_height = m_width, m_height
    menu_type = m_type
    
    font = love.graphics.newFont(12)
    
    calcd_menu_width = 0
    local new_w
    for k,v in pairs(menu_items) do
        new_w = font:getWidth(v[1])*3+20
        if calcd_menu_width < new_w then
            calcd_menu_width = new_w
        end
    end
    menu_width = calcd_menu_width
end

function menu_exit()
    --The exit callback should NOT be called when entering another state, 
    --only when explicitly exiting and falling back to the previous state.

    --Restore our callbacks when we're done.
    callbacks = deepcopy(previous_callbacks[#previous_callbacks])
    table.remove(previous_callbacks)
end

function menu_init()
    --Do your setup things here.
    --This should consist of things that only need to be called once per instance of the game.
    --Things like globals and such.
    --If anything needs to be done every time the state starts up, put it in menu_enter().
end

function menu_keypressed(key, unicode)
    if key == "return" then
        --Back out of this menu
        menu_exit()
    elseif key == " " then
        --Call the function
        menu_items[menu_current_selection][2](unpack(menu_items[menu_current_selection][3]))
    elseif key == "left" or key == "up" then
        if menu_current_selection-1 < #menu_items then
            menu_current_selection = #menu_items
        end
    end
    
    if debug_on then
        if key == "p" then
            --Pressed p, so do something, but only if debugging is on.
        end
    end
end

function menu_keyreleased(key, unicode)
    if key == " " then
        --They released space!
    end
end

function menu_update(dt)
    --Just like LOVE's update method.
    --In other words, whatever you want to run every frame.
end

function menu_draw()
    last_draw()
    --if cameracenter_translate then
        --love.graphics.translate(-cameracenter_translate[1], -cameracenter_translate[2])
    --end
    
    --Draw the frame
    love.graphics.setColor(255, 255, 255)
    love.graphics.rectangle("fill", menu_x, menu_y, menu_width+tilesize/2, menu_height)
    love.graphics.setColor(0, 0, 0)
    love.graphics.rectangle("line", menu_x, menu_y, menu_width+tilesize/2, menu_height)
    
    --Draw the text labels
    local xmod, ymod = tilesize/2, 0
    for k,v in pairs(menu_items) do
        if menu_type == "tall" then
            ymod = ymod + font:getHeight(v[1])*3
        end
        love.graphics.print(v[1], menu_x+xmod, menu_y+ymod)
        if k == menu_current_selection then
			love.graphics.draw(cached_load_image("/gfx/downarrow.png"), menu_x, menu_y+ymod, 0, 0.75, 0.75)
		end
    end
    
    --Draw the arrow indicator
    
    
end
