
require 'middleclass.init'

--Global table of all Mobiles
mobiles = {}

Mobile = class('Mobile')

--TODO: Move the physics out of Player and into here. 
--Mobiles will need physics unless otherwise explicitly stated.

--To make an animated, stationary Mobile: 
--  set self.moving to True as long as you want it to animate.
--  Then do nothing during its' thunk.
--  This class should be called "Animated" as opposed to "Mobile", and this should be "MobileAnimated".
--  Then this class's animation should be pulled out and put into a new class, and this one renamed.

function Mobile:initialize()
	--It's for Mobiles that dont move around on the map but do animate.
	--And no, we don't need a "Tile" or "Object" class. That'd be too slow I think.
	--Though an "InteractiveTile" class might be handy.
	
	self.name = "Mobile"
	self.direction = "down"
	self.moving = false
	self.worldx = 0
	self.worldxmult = 1
	self.worldy = 0
	self.worldymult = 1
    self.tilex = 0
    self.tiley = 0
	self.sprite = nil
	self.animation = nil
	self.map = ""
	self.physical = true
    self.speed = 64
    
    --Settings:
    self.sleep_when_offscreen = true
	
	--Insert ourself into the mobiles table when we're created.
	table.insert(mobiles, self)
    
    self:setAnim("gfx/physical_tile.png")
end

--Setters

function Mobile:setLoc(x, y)
	self.worldx = x
	self.worldy = y
    self.tilex = round(x/tilesize)
    self.tiley = round(y/tilesize)
end

function Mobile:setDir(dir)
	self.direction = dir
end

function Mobile:setMoving(moving)
	self.moving = moving
end

function Mobile:setAnim(filename)
	local img  = cached_load_image(filename)
	img.setFilter(img, "linear", "nearest")
	self.worldxmult = (tilesize*2)/img:getWidth()
	self.worldymult = (tilesize*2)/img:getHeight()
	local anim = newAnimation(img, img:getWidth()/2, img:getHeight()/2, player.down.fps/30, 0)

	self.animation = anim
end

function Mobile:load_animation(spritesheet, fps, tiles_height, tiles_width)
    local img = cached_load_image(spritesheet)
    img.setFilter(img, "linear", "nearest")
    self.worldxmult = (tilesize*2)/img:getWidth()
    self.worldymult = (tilesize*2)/img:getHeight()
    return newAnimation(img, img:getWidth()/tiles_height, img:getHeight()/tiles_width, fps/30, 0)
end

function Mobile:load_animations(filenameprefix, filenamesuffix, animSettingsObj)
    --Loads a set of animations given the filename prefix and suffix. 
    self.anims = {}
    for k,dir in pairs({"up", "down", "left", "right"}) do
        self.anims[dir] = self:load_animation(filenameprefix..dir..filenamesuffix,
                player[dir].fps,
                player[dir].tiles_height, 
                player[dir].tiles_width)
    end
    self.animation = self.anims[self.direction]
end

function Mobile:handlePhysics(dt, modx, mody)
    --Handle collisions
    
    if modx == nil or mody == nil then
        --Thunk didn't return position data, so we're not moving.
        return
    end
    
    -- Still a little 'tight' on 1-tile wide spaces, tends to get hung up on walls
    --TODO: Add a little code that shunts the player off to the side they're closest to to "funnel" the player into 1-tile wide gaps.
    local newmobx = self.worldx+modx
    local newmoby = self.worldy+mody
    local mob = {}
    mob.x = newmobx
    mob.y = newmoby
    mob.w = tilesize*0.95
    mob.h = tilesize*0.95
    
    local mobtilex, mobtiley
    local actionmobtilex, actionmobtiley
    
    mobtilex = round((newmobx/tilesize))
    mobtiley = round((newmoby/tilesize))
    
    --FIXME: This isnt quite right. We'll need more robust physics.
    --Physics
    if physics then
        --FIXME: Instead of looping all our physical tiles, use x and y as keys for a table.
        --  Then when we access the table, it's instantaneous. If anything is at [{x,y}], we just bail.
        --  No rect collision necessary.
        local p = physical_tiles_with_keys
        local x = mobtilex
        local y = mobtiley
        
        --First we'll check if there are physical tiles around the mobile.
        local handlephysics = false
        for k,v in pairs({-1, 0, 1}) do
            for l,w in pairs({-1, 0, 1}) do
                if p[x+v..','..y+w] ~= nil then
                    --If there are, handle physics like normal. (IE, slowly.)
                    handlephysics = true
                end --endif
            end --endfor
        end --endfor
        
        if handlephysics then
            for k,t in pairs(physical_tiles) do
                local tile = {}
                
                tile.x = t[1]*tilesize
                           tile.y = t[2]*tilesize
                tile.w = tilesize
                tile.h = tilesize
                
                -- If they collide with anything, dont move them.
                if colliderects(tile, mob) then
                    modx = 0
                    mody = 0
                    break
                end--endifcolliderects
            end --endfor
        end --endif
    end --endif physics
    
    self:setLoc((self.worldx+modx), (self.worldy+mody))
    return {modx, mody}
end

function Mobile:isVisible()
    local minx, maxx, miny, maxy
    minx = round((playerObj.worldx - (windowwidth/2))/tilesize)-1
    maxx = round((playerObj.worldx + (windowwidth/2))/tilesize)+1
    miny = round((playerObj.worldy - (windowheight/2))/tilesize)-1
    maxy = round((playerObj.worldy + (windowheight/2))/tilesize)+1
    
    local a, b
    a = self.tilex
    b = self.tiley
    
    if minx < a and maxx > a and miny < b and maxy > b then
        return true
    else
        return false
    end
end

--Public Methods

function Mobile:update(dt)
	--Do nothing by default.
	--Users of this class will have to override this with a function of their devising.
	--Or that func can go elsewhere.
    
    --TODO: Write a 'walkTo' method that instructs the mob to walk to that coordinate over a given period of time.
    
    --Mobiles don't update when the player can't see them.
    --To disable this, subclass Mobile, and override isVisible to return true constantly.
    if self.sleep_when_offscreen == true and self:isVisible() == false then
        return
    end
    
    local modx, mody = unpack(self:thunk(dt))
    
    modx, mody = unpack(self:handlePhysics(dt, modx, mody))
    
    --If we aren't moving, dont move. ;)
    if modx == 0 and mody == 0 then
        self.moving = false
    end
    
	self.animation = self.anims[self.direction]
	self.animation:update(dt)
    
	if self.moving == true then
		self.animation:play()
	else
		self.animation:seek(1)
        self.animation:stop()
	end
	
	
end

function Mobile:thunk(dt)
	--A placeholder method to be overridden to control what it does each thought cycle.
end

function Mobile:draw()
	self.animation:draw(self.worldx, self.worldy-16, 0, self.worldxmult, self.worldymult)
end

function make_default_mob()
	local mob = Mobile()
    mob:load_animations("gfx/player_", "_spritesheet.png", player)
end
