
require 'middleclass.init'

require("mobiles.lua")

Player = Mobile:subclass('Player')

function Player:initialize()
    Mobile.initialize(self)
	self.name = "Player"
	self.moving = false
	self.direction = "right"
	
end

function Player:teleport(targetmap, x, y, centercamera)
	--Loads a given targetmap, places the player at a given x/y coord and optionally centers the camera.
    print("Teleporting you (Via the player class) to...", targetmap, x, y, centercamera)
    load_map(gamesdir..gamename.."/maps/"..targetmap..".tmx")
    self.worldx = x*tilesize
    self.worldy = y*tilesize
    cameracenter = centercamera
    
    --Store some stuff so we can save it.
    player.map = targetmap
    player.locx = self.worldx
    player.locy = self.worldy
end

function Player:draw()
	love.graphics.draw(cached_load_image("/gfx/shadow.png"), self.worldx, self.worldy+10)
	self.animation:draw(self.worldx, self.worldy-16, 0, self.worldxmult, self.worldymult)
end

function Player:handleAnims(dt)
    
    self.animation = self.anims[self.direction]
    if self.moving then 
        self.animation:play() 
    end
    
	self.animation:update(dt)
	
-- 	if love.keyboard.isDown("up") or love.keyboard.isDown("down") or love.keyboard.isDown("left") or love.keyboard.isDown("right") then
-- 		if moving == true then
-- 			--This makes the player start to animate in the event that a key is pressed.
-- 			-- The net effect is that the player will kick their foot out when the key is just tapped.
-- 			-- This will be overridden with a seek to frame 1 if a key isnt pressed.
-- 			self.animation:seek(2) 
-- 		end
-- 	end
	
end

function Player:handleKeys(dt)
	local modx = 0
	local mody = 0
	
	if love.keyboard.isDown("up")  then
        self.moving = true
        mody = -speed * tilesize * dt
        
        if self.direction ~= "up" then
            self.direction = "up"
        end
    elseif love.keyboard.isDown("down")  then
        self.moving = true
        mody = speed * tilesize * dt
        
        if self.direction ~= "down" then
            self.direction = "down"
        end
    elseif love.keyboard.isDown("left")  then
        self.moving = true
        modx = -speed * tilesize * dt
        if self.direction ~= "left" then
            self.direction = "left"
        end
    elseif love.keyboard.isDown("right")  then
        self.moving = true
        modx = speed * tilesize * dt
        if self.direction ~= "right" then
            self.direction = "right"
        end
    else 
        self.moving = false
        self.animation:seek(1)
        self.animation:stop()
    end
    
    return {modx, mody}
end

function Player:update(dt)
    
    local modx, mody = unpack(self:handleKeys(dt))
    
    self:handleAnims(dt)
    
    self:handlePhysics(dt, modx, mody)
end

function make_player()
	local player = Player()
    player:load_animations("gfx/player_", "_spritesheet.png", player)
    return player
end
