
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.

--This file is all the love callbacks and functions for the walkabout (overworld) part of the game.

require("map_methods")
require("menu")

require("mobiles")
require("playerClass")

function load_player(spritesheet, fps, tiles_height, tiles_width)
    local img  = love.graphics.newImage(spritesheet)
    img.setFilter(img, "linear", "nearest")
    playerxmult = (tilesize*2)/img:getWidth()
    playerymult = (tilesize*2)/img:getHeight()
    
    return newAnimation(img, img:getWidth()/tiles_height, img:getHeight()/tiles_width, fps/30, 0)
end

maps = {} --All the layers of our maps preloaded

function load_map(filename)
    
    if maps[filename] == nil then
        layers, layerorder = load_tiled_map(filename)
        maps[filename] = {}
        maps[filename].layers = layers
        maps[filename].layerorder = layerorder
    else
        layers, layerorder = maps[filename].layers, maps[filename].layerorder
    end
        
    --Cache the physical tiles so we can collide against them easily later:
    physical_tiles = {}
    physical_tiles_with_keys = {}
    for k,l in pairs(layers) do --Foreach Map in layers
        for a,x in pairs(l) do --Foreach row in map
            if in_table(a, maplayer_properties) == false then
                for b,y in pairs(x) do --Foreach col in row
                    if y["physical"] ~= nil and y["physical"] ~= false then
                        table.insert(physical_tiles, {a, b})
                        physical_tiles_with_keys[a..","..b] = y
                    end
                    if type(y["image"]) == string then
                        y["image"] = eval(y["image"])
                    end
                    if y["onInit"] ~= nil then
                        locx = a
                        locy = b
                        eval(y["onInit"])
                    end
                end
            end
        end
    end
end

function walkabout_enter()
    --"Enters" the walkabout state
    --Sets up all the callbacks and such.
    
    table.insert(previous_callbacks, deepcopy(callbacks)) --Save the previous callbacks list
    
    callbacks["keypressed"] = walkabout_keypressed
    callbacks["keyreleased"] = walkabout_keyreleased
    callbacks["update"] = walkabout_update
    callbacks["draw"] = walkabout_draw
    callbacks["exit"] = walkabout_exit
end

function walkabout_exit()
    --The exit callback should NOT be called when entering another state, 
    --only when explicitly exiting and falling back to the previous state.

    print('exiting walkabout')
    
    --Restore our callbacks when we're done.
    callbacks = deepcopy(previous_callbacks[#previous_callbacks])
    table.remove(previous_callbacks)
end

function walkabout_init() --Should only be called once and from love.load()
    tilesize = 64
    speed = 3 --Movement speed on each keydown detection
    cameracenter = true
    
    physics = true
    events = true
    
    maplayer_properties = {"visible",}
    
    --load_map(gamesdir..gamename.."/maps/"..player.start.map..".tmx")
    
    anims = {}
    anims.down = load_player(player.down.filename, player.down.fps, player.down.tiles_height, player.down.tiles_width)
    anims.up = load_player(player.up.filename, player.up.fps, player.up.tiles_height, player.up.tiles_width)
    anims.left = load_player(player.left.filename, player.left.fps, player.left.tiles_height, player.left.tiles_width)
    anims.right = load_player(player.right.filename, player.right.fps, player.right.tiles_height, player.right.tiles_width)
    
    player_dir = player.start.direction
    
    moving = false
    
    --playerx = player.start.x*tilesize
    --playery = player.start.y*tilesize
    
    playerObj = make_player()
    playerObj:teleport(player.start.map, player.start.x, player.start.y, true)
    player.playerObj = playerObj
    
    --teleport(player.start.map, player.start.x, player.start.y, true)
    
    
    --TODO: Load player object from a save file (and save it too, for that matter)
    
    
    
    
end

function walkabout_keypressed(key, unicode)
    if key == " " then
        activate = true
        --TODO: Make this work with the playerObj
        local xmod, ymod = 0, 0
        local player_dir = playerObj.direction
        if player_dir == "down" then
            ymod = tilesize
        elseif player_dir == "up" then
            ymod = -tilesize
        elseif player_dir == "left" then
            xmod = -tilesize
        elseif player_dir == "right" then
            xmod = tilesize
        end
        print(playerObj.worldx+xmod)
        activate_loc = {playerObj.worldx+xmod, playerObj.worldy+ymod}
    elseif key == "return" then
        menu_enter({{"Index", print, {"hi there"}}, {"Wildmon", do_wild, {"poop"}}, {"Really ", print, {"hi there"}}}, "tall", 10, 10, (windowwidth/8)*3, (windowheight/4)*3)
    elseif key == "s" then
        --Save any walkabout things like player position, map name, and so on.
        
    elseif key == "l" then

    end
    
    if debug_on then
        if key == "p" then
            physics = not(physics)
        elseif key == "e" then
            events = not(events)
        elseif key == "x" then
            --Clear the map cache so maps are reloaded from disk.
            maps = {}
        elseif key == "w" then
            do_wild({})
        elseif key == "c" then
            --Choice!
            require("choice")
            choice_enter({{"YES", print}, {"NO", print}, {"FJK", print}, {"ERW", print}, {"RWW", print}, {"gds", print}}, "FJK", 20, 0, "left", "bottom")
        elseif key == "m" then
            --Mobs!
            local i = 0
            while i < 100 do
                setup_sample_mobile()
                i = i+1
            end
            print(#mobiles)
        end
    end
end

function walkabout_keyreleased(key, unicode)
    if key == " " then
        activate = false
    end
end

function walkabout_update(dt)
    --local modx, mody
    --modx = 0
    --mody = 0
    
    --anim = anims[player_dir]
    --anim:update(dt)
    
    --local framenum = anim:getCurrentFrame()
    
    --if moving == false then
        ----This makes the player start to animate in the event that a key is pressed.
        ---- The net effect is that the player will kick their foot out when the key is just tapped.
        ---- This will be overridden with a seek to frame 1 if a key isnt pressed.
        --anim:seek(2) 
    --end
    
    --Mobiles
    for k,mob in pairs(mobiles) do
		mob:update(dt)
    end
    
    --if love.keyboard.isDown("up")  then
        --moving = true
        --mody = -speed * tilesize * dt
        
        --if player_dir ~= "up" then
            --anim = anims.up
            --player_dir = "up"
        --end
        
        --anim:play()
    --elseif love.keyboard.isDown("down")  then
        --moving = true
        --mody = speed * tilesize * dt
        
        --if player_dir ~= "down" then
            --anim = anims.down
            --player_dir = "down"
        --end
        
        --anim:play()
    --elseif love.keyboard.isDown("left")  then
        --moving = true
        --modx = -speed * tilesize * dt
        --if player_dir ~= "left" then
            --anim = anims.left
            --player_dir = "left"
        --end
        --anim:play()
    --elseif love.keyboard.isDown("right")  then
        --moving = true
        --modx = speed * tilesize * dt
        --if player_dir ~= "right" then
            --anim = anims.right
            --player_dir = "right"
        --end
        --anim:play()
    --else 
        --moving = false
        --anim:seek(1)
        --anim:stop()
    --end
    
    --Handle collisions
    -- Still a little 'tight' on 1-tile wide spaces, tends to get hung up on walls
    --TODO: Add a little code that shunts the player off to the side they're closest to to "funnel" the player into 1-tile wide gaps.
    --local newplayerx = playerx+modx
    --local newplayery = playery+mody
    --local player = {}
    --player.x = playerx+modx
    --player.y = playery+mody
    --player.w = tilesize*0.95
    --player.h = tilesize*0.95
    
    --local playertilex, playertiley
    --Might need something more robust here.
    --FIXME: This isnt quite right. We'll need more robust physics.
    --playertilex = round(newplayerx/tilesize)
    --playertiley = round(newplayery/tilesize)
    
    --TODO: In each for loop, if the tile we're looking at is obviously out of range, skip it.
    
    --Physics
    --if physics then
        --for k,t in pairs(physical_tiles) do
            --local tile = {}
            
            --tile.x = t[1]*tilesize
            --tile.y = t[2]*tilesize
            --tile.w = tilesize
            --tile.h = tilesize
            
            ---- If they collide with anything, dont move the player.
            --if colliderects(tile, player) then
                --modx = 0
                --mody = 0
                --break
            --end
        --end
    --end
    
    local actionplayertilex, actionplayertiley
    if activate == true then
        actionplayertilex = round(activate_loc[1]/tilesize)
        actionplayertiley = round(activate_loc[2]/tilesize)
    end
    
    --Events:
    local playertilex = playerObj.tilex
    local playertiley = playerObj.tiley
    if events then
        for k,t in pairs(layers) do
                if t[playertilex] ~= nil then
                    local thistile = t[playertilex][playertiley]
                    if thistile ~= nil then
                        -- Fire the events in the tile we're on. 
                        -- onStep events will fire exactly once when the player enters that tile.
                        if thistile["onStep"] ~= nil and (lastplayertilex ~= playertilex or lastplayertiley ~= playertiley) then
                            eval(thistile["onStep"])
                        end
                    end
                    
                    local thisactiontile
                    if activate == true then
                        thisactiontile = t[actionplayertilex][actionplayertiley]
                    end
                    if thisactiontile ~= nil then
                        --Fire the events in the tile we're on _when activated_.
                        --TODO: Make activating only for the tile in front of the player.
                        --If they're more than half way into a tile, it's the next one. If they're less, it's the one they're on.
                        if thisactiontile["onActivate"] ~= nil and activate == true then
                            eval(thisactiontile["onActivate"])
                            activate = false
                        end
                    end
                end
        end
    end
    
    lastplayertilex = playertilex
    lastplayertiley = playertiley
    
    --playerx = playerx + modx
    --playery = playery + mody
    
--~     --Handle our mapmethod_queue
--~     if mapmethod_queue_lock == false then
--~         mapmethod_queue[1](unpack(mapmethod_queue[2]))
--~         table.remove(mapmethod_queue, 1)
--~     end
    
end

function walkabout_draw()
	love.graphics.push()
	
    --Center our camera
    if cameracenter == true then
        cameracenter_translate = {(windowwidth/2)-(playerObj.worldx)-(tilesize/2), (windowheight/2)-(playerObj.worldy)-(tilesize/2)}
        love.graphics.translate((windowwidth/2)-(playerObj.worldx)-(tilesize/2), (windowheight/2)-(playerObj.worldy)-(tilesize/2))
    end
    
    --Calculate the minimum and maximum X and Y tile coordinate we can see and store it for use later in this loop
    local minx, maxx, miny, maxy
    minx = round((playerObj.worldx - (windowwidth/2))/tilesize)-1
    maxx = round((playerObj.worldx + (windowwidth/2))/tilesize)+1
    miny = round((playerObj.worldy - (windowheight/2))/tilesize)-1
    maxy = round((playerObj.worldy + (windowheight/2))/tilesize)+1
    
    --TODO: Add recognition of layers that are above the player so we can draw them on top of the player.
    --Draw our tiles
    for k,v in pairs(layerorder) do --Foreach Map in layers
        l = layers[v]
        if l.visible ~= nil and l.visible ~= false then --If the map layer's is invisible
            for a,x in pairs(l) do --Foreach row in map
                if in_table(a, maplayer_properties) == false then
                    for b,y in pairs(x) do --Foreach col in row
                        --Skip this tile if it's not visible in the window.
                        if minx < a and maxx > a and miny < b and maxy > b then
                            --TODO: Could break this part out into a Tiles class.
                            local tile = {}
                            tile.x = (a)*tilesize
                            tile.y = (b)*tilesize
                            tile.w = tilesize
                            tile.h = tilesize
                            if y.image ~= nil then
                                --y.image might be a snippet of lua code to load an image.
                                --It will define a local, "thisimg", which will be what draw expects.
                                if type(y.image) == "string" then
                                    eval(y.image, true)
                                    if thisimg ~= nil then
                                        y.image = thisimg
                                        thisimg = nil
                                    end
                                end
                                if type(y.image) == "userdata" then --Probably an Image.
                                    love.graphics.draw(y.image, tile.x, tile.y, 0, y.mult, y.mult)
                                elseif type(y.image) == "table" then --Probably an AnAL animation.
                                    y.image:draw(tile.x, tile.y, 0, y.mult, y.mult)
                                end
                            end
                        end
                    end
                end
            end
        end
        
        --Mobiles will be drawn above the middle layer.
        if k == round((#layerorder/2)+0.5) then
            -- Now draw mobiles. The player is also a mobile.
            for k,mob in pairs(mobiles) do
                --We draw all the mobs here because they're super fast to draw.
                --And that the longest thing about mobs is the updating, which only happens
                --  when they are within view.
                mob:draw()
            end
        end
    end
    

    
    --Draw some outlines for our physical tiles and wilderness tiles for debug purposes.
    if debug_on then
        love.graphics.setLineWidth(10)
        
        --Draw a rect for physical tiles
        if physics then
            love.graphics.setColor(255, 128/2, 128/2)
            for k,t in pairs(physical_tiles) do
                local tile = {}
                tile.x = t[1]*tilesize
                tile.y = t[2]*tilesize
                tile.w = tilesize
                tile.h = tilesize
                love.graphics.rectangle("line", tile.x, tile.y, tilesize, tilesize)
            end
        end
        
        if events then
            for k,l in pairs(layers) do --Foreach Map in layers
                for a,x in pairs(l) do --Foreach row in map
                    if in_table(a, maplayer_properties) == false then
                        for b,y in pairs(x) do --Foreach col in row
                            --Draw a rect for tiles with events
                            if y["onStep"] ~= nil or y["onActivate"] ~= nil then
                                love.graphics.setColor(128/2, 128/2, 255)
                                --Blue for something
                                love.graphics.rectangle("line", a*tilesize, b*tilesize, tilesize, tilesize)
                            end
                            
                            --Draw a rect for tiles with events that are wild tiles
                            if y["onStep"] ~= nil and  string.find(y["onStep"], "do_wild") ~= nil then
                                love.graphics.setColor(128/2, 255, 128/2)
                                love.graphics.rectangle("line", a*tilesize, b*tilesize, tilesize, tilesize)
                            end
                        end
                    end
                end
            end
        end
        love.graphics.setColor(255, 255, 255)
    end
    
    love.graphics.pop()
end
