
--Copyright 2010 Gryc Ueusp

--This file is part of Collectible RPG Engine.

--Collectible RPG Engine is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.

--Collectible RPG Engine is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.

--You should have received a copy of the GNU General Public License
--along with Collectible RPG Engine.  If not, see <http://www.gnu.org/licenses/>.


require("dialog")

--This file handles all the dialogs (speech and other text) that you see in the walkabout state.
--It captures keyboard events and does not pass them to the walkabout state.
--It also calls the walkabout's draw function before its own to draw the tile backgrounds.

local last_draw, current_message, exit_method, exit_args

local blinktime = 0
local blink = false

function walkabout_dialog_enter(callback, callback_args)
    --"Enters" the state
    --Sets up all the callbacks and such.
    --Call this when you want to move to this state.
    --Including a callback will cause that to be called when the state exits
    
    exit_method, exit_args  = callback, callback_args
    
    table.insert(previous_callbacks, deepcopy(callbacks)) --Save the previous callbacks list
    
    callbacks["keypressed"] = walkabout_dialog_keypressed
    callbacks["keyreleased"] = walkabout_dialog_keyreleased
    callbacks["update"] = walkabout_dialog_update
    last_draw = callbacks["draw"]
    callbacks["draw"] = walkabout_dialog_draw
    callbacks["exit"] = walkabout_dialog_exit
    
    current_message = queued_dialogs[#queued_dialogs]
    table.remove(queued_dialogs) --Pop
    
end

function walkabout_dialog_exit()
    --The exit callback should NOT be called when entering another state, 
    --only when explicitly exiting and falling back to the previous state.
    
    --Restore our callbacks when we're done.
    print("exiting walkabout dialog")
    callbacks = deepcopy(previous_callbacks[#previous_callbacks])
    table.remove(previous_callbacks)
    
    if exit_method ~= nil then
        if type(exit_method) == "table" then
            print_table(exit_method)
        end
        if exit_args ~= nil then
            exit_method(unpack(exit_args))
        else
            exit_method()
        end
        
    end
    
end

function walkabout_dialog_init()
    --Do your setup things here.
    --This should consist of things that only need to be called once per instance of the game.
    --Things like globals and such.
    --If anything needs to be done every time the state starts up, put it in walkabout_dialog_enter().
    
    love.graphics.setFont(love.graphics.newFont(tilesize/2))
    
    frame = load_dialog_frame("/gfx/frames/fantasy_bold.png")
    
end

function walkabout_dialog_keypressed(key, unicode)
    if key == " " then
        --Pressed space, so do something if you want.
        
        if #queued_dialogs > 0 then
            current_message = queued_dialogs[1]
            table.remove(queued_dialogs, 1)
        
        --Done showing messages, so exit the dialog.
        elseif #queued_dialogs == 0 then
            print("out of messages")
            callbacks.exit()
        end
    end
    
    if debug_on then
        if key == "p" then
            --Pressed p, so do something, but only if debugging is on.
            print("pressed p while in the walkabout dialog state!")
        end
    end
end

function walkabout_dialog_keyreleased(key, unicode)
    if key == " " then
        --They released space!
    end
end

function walkabout_dialog_update(dt)
    blinktime = blinktime + dt
    if blinktime > 0.5 then
        blinktime = 0
        blink = not(blink)
    end
    --Just like LOVE's update method.
    --In other words, whatever you want to run every frame.
end

function walkabout_dialog_draw()
    last_draw()
        
    --TODO: Move all this math into the init function.
    
    local draw_retval = {draw_dialog_frame(frame, 0, tilesize*7.5, windowwidth, windowheight-(tilesize*7.5))}
    local corner_height, corner_width, corner_sy, corner_sx, dialog_x, dialog_y, dialog_w, dialog_h, horizscale, vertscale = unpack(draw_retval)
    --And finally our text:
    love.graphics.setColor(255, 255, 255)
    local lineheight = love.graphics.getFont():getLineHeight()
    local linewidth = love.graphics.getFont():getWidth(current_message)
--~         if linewidth > tilesize*10 then
--~             --TODO: Split line so that we can have automatic wordwrapping.
--~             print("WARNING: DIALOG TEXT IS "..tonumber(linewidth-(tilesize*10)).." PIXELS TOO LONG.  ADD SOME NEWLINES: ", current_message)
--~         end
    love.graphics.print(current_message, corner_width*corner_sx*1.25, dialog_y+(corner_height*corner_sy*3))
    
    --Oh, and a little blinky continuation marker.
    if blink == true then
        love.graphics.draw(cached_load_image("/gfx/downarrow.png"), windowwidth-tilesize-(corner_width/2), windowheight-tilesize-(corner_height/2), 0, 0.75, 0.75)
    end
        
    --Maybe skip the whole UI library altogether and just draw our corners and sides manually
    --And then draw some text.
    --We'd need some logic up in the update function to break text into separate dialogs.
    
    
end
